<?xml version="1.0" encoding="UTF-8"?>
<!--
/**
 * UCMExporter
 * @author Ali Echihabi
 *
 * (C) Copyright 2003 Use Case Maps User Group
 * www.usecasemaps.org
 */


	This XSL sheet is to be applied to the XML file corresponding to the UCMExporterXML.dtd
	in order to generate the corresponding MSC for each scenario.
	UCMExporter can transform the XML generated by UCMNav to the UCMExporterXML.dtd.
-->
<!-- start of xsl sheet -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xsltc="http://xml.apache.org/xalan/xsltc"
	extension-element-prefixes="xsltc" version="1.0">
<xsl:output method = "text" />
  <!-- By default output to the current directory-->
  <xsl:param name="OutputDirectory"/>


<xsl:template match="/">

	<!-- generate MSC for each scenario -->
	<xsl:for-each select="//scenario">
		<!-- generate the corresponding MSC -->
		<xsl:call-template name="generateScenarioMSC"/>
	</xsl:for-each>

</xsl:template>




<xsl:template name="generateScenarioMSC">
<xsltc:output file="{$OutputDirectory}\{@name}.msc">
		
		<!-- put the msc header -->
		<xsl:call-template name="putHeader"/>		
		
		<!-- list participating instances -->
		<xsl:text>&#xa;</xsl:text>
		<xsl:call-template name="listScenarioInstances"/>
		
		<!-- do message sequence -->
		<xsl:text>&#xa;</xsl:text>
		<xsl:call-template name="doMessageSequence"/>		

		<!-- terminate instances -->	
		<xsl:text>&#xa;</xsl:text>
		<xsl:call-template name="terminateScenarioInstances"/>
		
		<!-- put the msc footer -->
		<xsl:text>&#xa;</xsl:text>
		<xsl:call-template name="putFooter"/>		
		
</xsltc:output>	
</xsl:template>




<xsl:template name="putHeader">
	<xsl:text>&#xa;</xsl:text>
	<xsl:text>mscdocument </xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>;</xsl:text>
		
	<xsl:text>&#xa;</xsl:text>
	<xsl:text>msc </xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>;</xsl:text>
</xsl:template>




<xsl:template name="listScenarioInstances">
	<xsl:text>&#xa;</xsl:text>
	<xsl:for-each select=".//instance">
		<xsl:call-template name="getScenarioInstanceNameFromID">
			<xsl:with-param name="id" select="@id"/>
		</xsl:call-template>
		<xsl:text> :instance;</xsl:text>
		<xsl:text>&#xa;</xsl:text>
	</xsl:for-each>	
</xsl:template>




<xsl:template name="terminateScenarioInstances">
	<xsl:text>&#xa;</xsl:text>
	<xsl:for-each select=".//instance">
		<xsl:call-template name="getScenarioInstanceNameFromID">
			<xsl:with-param name="id" select="@id"/>
		</xsl:call-template>
		<xsl:text> :endinstance;</xsl:text>
		<xsl:text>&#xa;</xsl:text>
	</xsl:for-each>
</xsl:template>



<xsl:template name="getScenarioInstanceNameFromID">
	<xsl:param name="id" select="'I0'"/>
	<xsl:value-of select="ancestor::scenario/instances/instance[@id=$id]/@name"/>
	<!--<xsl:value-of select="@name"/>-->
</xsl:template>


<xsl:template name="putFooter">
	<xsl:text>&#xa;</xsl:text>
	<xsl:text>endmsc;</xsl:text>
</xsl:template>




<xsl:template name="doMessageSequence">

	<!-- handle each message sequence/parallel block recursively
		 apply the one that comes first
	 -->
	<!-- <xsl:apply-templates select="child::seq | child::par"/> -->
		<!-- apply the defined templates -->
		<xsl:apply-templates/>
	
</xsl:template>




<xsl:template name="doSequenceBlock" match="seq">

	<!-- handle messages and conditions in this sequence block -->
	
<!--	<xsl:apply-templates select="child::message | child::condition "/>
	-->



	<!-- process any sub-message sequences (seq or par) -->
	<xsl:call-template name="doMessageSequence"/>
</xsl:template>



<xsl:template name="doParallelBlock" match="par">
	<xsl:text>&#xa;</xsl:text>
	<xsl:text>all: par begin;</xsl:text>

	<xsl:for-each select="child::seq">
		<xsl:call-template name="doSequenceBlock"/>
		<xsl:if test="position() != last()">
			<xsl:text>&#xa;</xsl:text>
			<xsl:text>par;</xsl:text>
			<xsl:text>&#xa;</xsl:text>
		</xsl:if>
	</xsl:for-each>

	<xsl:text>&#xa;</xsl:text>
	<xsl:text>par end;</xsl:text>

</xsl:template>


<xsl:template match="message">
	<!-- handle message depending on whether it a task, a simple message, a timer... -->

	<xsl:choose>
		
		<xsl:when test="@is-task = 'true'">
			<!-- handle as task -->
			<xsl:call-template name="showTaskMessage"/>
		</xsl:when>
		
		<xsl:when test="@is-timer = 'true'">
			<!-- handle as a timer-related message -->
			<xsl:call-template name="showTimerRelatedMessage"/>
		</xsl:when>
		<xsl:otherwise>
			<!-- a simple message -->
			<xsl:call-template name="showSimpleMessage"/>
		</xsl:otherwise>
		
	</xsl:choose>	
	

</xsl:template>




<xsl:template name="showTaskMessage">
	<xsl:text>&#xa;</xsl:text>
	<xsl:call-template name="getScenarioInstanceNameFromID">
		<xsl:with-param name="id" select="@source-id"/>
	</xsl:call-template>
	<xsl:text>: </xsl:text>
	<xsl:text>action '</xsl:text>
	<xsl:text>TASK </xsl:text>
	<xsl:value-of select="@name"/>	
	<xsl:text>'; </xsl:text>
</xsl:template>



<xsl:template name="showTimerRelatedMessage">
	
	<xsl:if test="@timer-property = 'set'">
		<!-- for timer sets, put a task then set the timer -->
		<xsl:call-template name="showTaskMessage"/>
		
		<xsl:text>&#xa;</xsl:text>
		<xsl:call-template name="getScenarioInstanceNameFromID">
			<xsl:with-param name="id" select="@source-id"/>
		</xsl:call-template>
		<xsl:text>: </xsl:text>
		<xsl:text> set </xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>, </xsl:text>
		<xsl:value-of select="@name"/>	
		<xsl:text>; </xsl:text>
	</xsl:if>
	
	<xsl:if test="@timer-property = 'reset'">
		<xsl:text>&#xa;</xsl:text>
		<xsl:call-template name="getScenarioInstanceNameFromID">
			<xsl:with-param name="id" select="@source-id"/>
		</xsl:call-template>
		<xsl:text>: </xsl:text>
		<xsl:text> reset </xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>, </xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>; </xsl:text>
	</xsl:if>
	

</xsl:template>



<xsl:template name="showSimpleMessage">

	<!-- show message as "out" -->
	<xsl:text>&#xa;</xsl:text>
	<xsl:call-template name="getScenarioInstanceNameFromID">
		<xsl:with-param name="id" select="@source-id"/>
	</xsl:call-template>
	<xsl:text>: </xsl:text>
	<xsl:text>out </xsl:text>
	<xsl:value-of select="@name"/>	
	<!-- I though that any thing could go here, nope! It has to be unique...at least in the immediate scope of the message-->
	<xsl:text>, </xsl:text>
	<xsl:value-of select="@id"/>
	<xsl:text> to </xsl:text>
	<xsl:call-template name="getScenarioInstanceNameFromID">
		<xsl:with-param name="id" select="@destination-id"/>
	</xsl:call-template>
	<xsl:text>; </xsl:text>
	
	<!-- show messsage as "in" -->
	<xsl:text>&#xa;</xsl:text>
	<xsl:call-template name="getScenarioInstanceNameFromID">
		<xsl:with-param name="id" select="@destination-id"/>
	</xsl:call-template>
	
	<xsl:text>: </xsl:text>
	<xsl:text>in </xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>, </xsl:text>
	<xsl:value-of select="@id"/>
	<xsl:text> from </xsl:text>
	<xsl:call-template name="getScenarioInstanceNameFromID">
		<xsl:with-param name="id" select="@source-id"/>
	</xsl:call-template>
	<xsl:text>; </xsl:text>
</xsl:template>




<xsl:template match="condition">
	<xsl:text>&#xa;</xsl:text>
	
	
	<xsl:call-template name="getScenarioInstanceNameFromID">
		<xsl:with-param name="id" select="@instance"/>
	</xsl:call-template>

	<xsl:text>: condition </xsl:text>
	<xsl:value-of select="@label"/>
	<xsl:text>;</xsl:text>
</xsl:template>


</xsl:stylesheet>

	
<!-- end of xsl sheet -->