<?xml version="1.0"?>
<!-- scenarios1.xsl - this stylesheet performs transformations from UCM scenarios
	to TTCN3 code..
	Author: Bryan Mulvihill
	Date: March 1, 2003
	Last Updated: June 17, 2003 by Ali Echihabi
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xsltc="http://xml.apache.org/xalan/xsltc"
	extension-element-prefixes="xsltc" version="1.0">
  <xsl:output method = "text" />
  <!-- By default output to the current directory-->
  <xsl:param name="OutputDirectory"/>

<!--
	group
		- creates any default components/type that need to be created
		- creates functions for each parallel test component that needs
		creating.
-->
  <xsl:template match="group">
  	<xsltc:output file="{$OutputDirectory}\{@name}.mlf">&lt;?xml version="1.0"?&gt;
&lt;!DOCTYPE moduleloader PUBLIC "-//TESTING TECH//DTD MLF//1.4" "mlf.dtd"&gt;
&lt;moduleloader&gt;
  &lt;module File="<xsl:value-of select="@name"/>.jar" Name="<xsl:value-of select="@name"/>" Package=""&gt;

    &lt;testadapter File="" Name="UCMTestAdapter"&gt;
    	&lt;description&gt;Testadapter for UCM&lt;/description&gt;
&lt;/testadapter&gt;

	<xsl:for-each select="scenario">
&lt;testcase Name="<xsl:value-of select="@name"/>" Selection="true" Status="stopped" Verdict="none"&gt;
	&lt;description&gt;<xsl:value-of select="@description"/>&lt;/description&gt;
&lt;/testcase&gt;
	</xsl:for-each>
&lt;/module&gt;
&lt;/moduleloader&gt;
  	</xsltc:output>
  	<xsltc:output file="{$OutputDirectory}\{@name}.ttcn3">
  	<xsl:text>module </xsl:text><xsl:value-of select="@name"/><xsl:text>
{
	/*
	</xsl:text><xsl:value-of select="@description"/>
	<xsl:text>
	*/
	//Port type definition
	type port MixedPort mixed
	{
		inout all
	}
	</xsl:text>
	<xsl:for-each select="//timer">
	<xsl:text>timer </xsl:text><xsl:value-of select="@name"/><xsl:text>;
	</xsl:text>
	</xsl:for-each>

	<xsl:for-each select="//component">
	<xsl:variable name="cname" select="@name"/>
	<xsl:for-each select="function">
	<xsl:text>signature </xsl:text><xsl:value-of select="$cname"/>00_00<xsl:value-of select="@name"/><xsl:text>() return integer;
	</xsl:text>
	</xsl:for-each>
	</xsl:for-each>

	<xsl:text>
	// Default Component
	type component DefaultComponent
	{
		port MixedPort thePort;
	}

	type component DefaultPTC
	{
		port MixedPort thePort;
	}
	</xsl:text>
	<!--Create Parallel test components-->
	<xsl:for-each select="//par/seq">
	<xsl:text>
	function PTC</xsl:text><xsl:value-of select="generate-id()"/><xsl:text>() runs on DefaultPTC
	{</xsl:text>
	<xsl:apply-templates/>
	<xsl:text>
	};</xsl:text>
  	</xsl:for-each>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
    </xsltc:output>
  </xsl:template>

<!--
	scenario - creates a testcase in its own name
-->
  <xsl:template match="scenario">
	<xsl:text>testcase </xsl:text><xsl:value-of select="@name"/><xsl:text>()
	 runs on DefaultComponent system DefaultComponent
	{
		map(mtc:thePort, system:thePort);
	</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>
    	verdict.set(pass);	//by default everything passes
    }</xsl:text>
  </xsl:template>

<!--
	par this creates a parallel test component for each child seq element
-->
  <xsl:template match="par">
	<xsl:for-each select="seq">
	<xsl:variable name="ptcName">PTC<xsl:value-of select="generate-id()"/></xsl:variable>
		<xsl:text>var DefaultPTC v</xsl:text><xsl:value-of select="$ptcName"/><xsl:text>:= DefaultPTC.create;
		v</xsl:text><xsl:value-of select="$ptcName"/><xsl:text>.start( </xsl:text><xsl:value-of select="$ptcName"/><xsl:text>() );
		</xsl:text>
	</xsl:for-each>
		<xsl:for-each select="seq">
		<xsl:variable name="ptcName">vPTC<xsl:value-of select="generate-id()"/></xsl:variable>
		<xsl:value-of select="$ptcName"/><xsl:text>.done;
		</xsl:text>
	</xsl:for-each>

  </xsl:template>

<!--
	do templates - based on the type attribute
-->
  <xsl:template match="do[@type='Resp']">
  <xsl:variable name="func"><xsl:value-of select="@component-name"/>00_00<xsl:value-of select="@name"/></xsl:variable>
    thePort.call(<xsl:value-of select="$func"/><xsl:text>);
	alt {
		[] thePort.getreply(</xsl:text><xsl:value-of select="$func"/><xsl:text> value 0)	{ }
		[] thePort.getreply	{	//non zero value, fail
			verdict.set(fail);
		}
	}
    </xsl:text>
  </xsl:template>

  <xsl:template match="do[@type='Start']">
    <xsl:text>//Start</xsl:text>
  </xsl:template>

  <xsl:template match="do[@type='Timer_Set']">
    <xsl:value-of select="@name"/><xsl:text>.start;</xsl:text>
  </xsl:template>

  <xsl:template match="do[@type='Timer_Reset']">
      <xsl:value-of select="@name"/><xsl:text>.stop;</xsl:text>
  </xsl:template>

  <xsl:template match="do[@type='Timeout']">
      <xsl:value-of select="@name"/><xsl:text>.timeout;</xsl:text>
  </xsl:template>
  
<!--

-->
  <xsl:template match="condition">
  	<!--find out where conditions end-->
  </xsl:template>

  <xsl:template match="component">
  	<!--<xsltc:output file="{$OutputDirectory}\{@name}.java">-->
  	<xsl:text>public class </xsl:text><xsl:value-of select="@name"/><xsl:text>
  	{
  	</xsl:text>
  	<xsl:apply-templates/>
  	<xsl:text>
  	}</xsl:text>
  	<!--</xsltc:output>-->
  </xsl:template>

	<xsl:template match="function">
	public static int <xsl:value-of select="@name"/>()
	{
		return 0;
	}
	</xsl:template>
</xsl:stylesheet>