<?xml version="1.0" encoding="UTF-8"?>
<!--
/**
 * UCMExporter
 * @author Ali Echihabi
 *
 * (C) Copyright 2003 Use Case Maps User Group
 * www.usecasemaps.org
 */


	This XSL sheet is to be applied to the XML file corresponding to the UCMExporterXML.dtd
	in order to generate the corresponding Sequence Diagram in XMI format.
	UCMExporter can transform the XML generated by UCMNav to the UCMExporterXML.dtd.
-->
<!-- start of xsl sheet -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xsltc="http://xml.apache.org/xalan/xsltc"
	extension-element-prefixes="xsltc" version="1.0">
	<xsl:variable name="nsUML" select="'href://org.omg/UML/1.3'"/>
	<xsl:variable name="nsModel" select="'M0'"/>


	<!--


-->
	<xsl:template match="/">
		<xsl:element name="XMI">
			<xsl:element name="XMI.header">
				
				<!-- put xmi header -->
				<xsl:call-template name="putHeader"/>
			</xsl:element>
			
			<xsl:element name="XMI.content">
				<xsl:element name="UML:Model" use-attribute-sets="modelAttributes" namespace="{$nsUML}">
					<xsl:call-template name="putModel"/>
				</xsl:element>
			</xsl:element>
		</xsl:element>
	</xsl:template>

<!--


-->
<xsl:template name="putHeader">
	
	<xsl:element name="XMI.documentation">
		<xsl:element name="XMI.exporter">
			<xsl:text>UCMExporter</xsl:text>
		</xsl:element> 	
		<xsl:element name="XMI.exporterVersion">
			<xsl:text>1.1</xsl:text>
		</xsl:element>
	</xsl:element>

	<xsl:element name="XMI.metamodel">
		<xsl:attribute name="xmi.name">
			<xsl:text>UML</xsl:text>
		</xsl:attribute>
		<xsl:attribute name="xmi.version">
			<xsl:text>1.3</xsl:text>
		</xsl:attribute>
	</xsl:element>
		
</xsl:template>

<!--


-->
	<xsl:template name="putModel">
		<xsl:element name="UML:Namespace.ownedElement" namespace="{$nsUML}">
			<!-- insert collaborations -->
			<xsl:call-template name="putCollaborations"/>
			
			<!-- insert classes -->
			<xsl:call-template name="putClasses"/>
		</xsl:element>
	</xsl:template>
	<!--


-->
	<xsl:template name="putClasses">
		<!-- for each component add a class element -->
		<xsl:for-each select="/scenarios/components/child::*">
			<xsl:element name="UML:Class" use-attribute-sets="classAttributes" namespace="{$nsUML}"/>
		</xsl:for-each>
	</xsl:template>

<!--


-->
	<xsl:template name="putCollaborations">
		<xsl:element name="UML:Collaboration" namespace="{$nsUML}">
			<xsl:element name="UML:Namespace.ownedElement" namespace="{$nsUML}">
				<!-- put interaction objects (classifier roles)-->
				<xsl:call-template name="putInteractionObjects"/>
			</xsl:element>
			<xsl:element name="UML:Collaboration.interaction" namespace="{$nsUML}">
				<!-- put interactions and their messages-->
				<xsl:call-template name="putInteractions"/>
			</xsl:element>
		</xsl:element>
	</xsl:template>

<!--


-->
	<xsl:template name="putInteractionObjects">
		<!-- for each scenario (i.e interaction) , list the objects (roles) -->
		<!-- put an Association object to mark the end of the objects for the current interaction-->
		<xsl:for-each select="//scenario">
			<xsl:for-each select="instances/instance">
				<xsl:element name="UML:ClassifierRole" use-attribute-sets="classifierRoleAttributes" namespace="{$nsUML}"/>
			</xsl:for-each>
			<xsl:element name="UML:AssociationRole" namespace="{$nsUML}"></xsl:element>
		</xsl:for-each>
	</xsl:template>

<!--


-->
	<xsl:template name="putInteractions">
		<xsl:for-each select="//scenario">
			<xsl:element name="UML:Interaction" use-attribute-sets="interactionAttributes" namespace="{$nsUML}">
				<xsl:element name="UML:Interaction.message" namespace="{$nsUML}">
					<!-- put interaction messages -->
					<xsl:call-template name="putInteractionMessages"/>
				</xsl:element>
			</xsl:element>
		</xsl:for-each>
	</xsl:template>
	
	<!--


	-->
	<xsl:template name="putInteractionMessages">
		<!-- process all <messsage> elements in the current scenario -->
		<xsl:for-each select=".//message">
			<xsl:element name="UML:Message" use-attribute-sets="messageAttributes" namespace="{$nsUML}"/>
			<xsl:text> </xsl:text>
		</xsl:for-each>
	</xsl:template>

<!--



							XMI Attribute Sets 				




-->
	<xsl:attribute-set name="modelAttributes">
		<xsl:attribute name="xmi.id">
			<xsl:value-of select="$nsModel"/>
		</xsl:attribute>
		<xsl:attribute name="name">
			<xsl:value-of select="@name"/>
		</xsl:attribute>
		<xsl:attribute name="visibility">
			<xsl:value-of select="'public'"/>
		</xsl:attribute>
		<xsl:attribute name="isSpecification">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
		<xsl:attribute name="isRoot">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
		<xsl:attribute name="isLeaf">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
		<xsl:attribute name="isAbstract">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
	</xsl:attribute-set>
	
	<xsl:attribute-set name="classAttributes">
		<xsl:attribute name="xmi.id">
			<xsl:value-of select="@id"/>
		</xsl:attribute>
		<xsl:attribute name="name">
			<xsl:value-of select="@name"/>
		</xsl:attribute>
		<xsl:attribute name="visibility">
			<xsl:value-of select="'public'"/>
		</xsl:attribute>
		<xsl:attribute name="isSpecification">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
		<xsl:attribute name="isRoot">
			<xsl:value-of select="'true'"/>
		</xsl:attribute>
		<xsl:attribute name="isLeaf">
			<xsl:value-of select="'true'"/>
		</xsl:attribute>
		<xsl:attribute name="isAbstract">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
		<xsl:attribute name="isActive">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
		<xsl:attribute name="namespace">
			<xsl:value-of select="$nsModel"/>
		</xsl:attribute>
	</xsl:attribute-set>

	<!--


	-->
	<xsl:attribute-set name="interactionAttributes">
		<xsl:attribute name="xmi.id">
			<xsl:value-of select="concat('Interaction_', @name)"/>
		</xsl:attribute>
		<xsl:attribute name="name">
			<xsl:value-of select="@name"/>
		</xsl:attribute>
		<xsl:attribute name="visibility">
			<xsl:value-of select="'public'"/>
		</xsl:attribute>
		<xsl:attribute name="isSpecification">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
	</xsl:attribute-set>
	
	<!--


	-->
	<xsl:attribute-set name="classifierRoleAttributes">
		<!-- ClassifierRole_ + ID of instance -->
		<xsl:attribute name="xmi.id">
			<xsl:value-of select="concat('ClassifierRole_', @id)"/>
		</xsl:attribute>
		<xsl:attribute name="name">
			<xsl:value-of select="@name"/>
		</xsl:attribute>
		<xsl:attribute name="visibility">
			<xsl:value-of select="'public'"/>
		</xsl:attribute>
		<xsl:attribute name="isSpecification">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
		<xsl:attribute name="isRoot">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
		<xsl:attribute name="isLeaf">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
		<xsl:attribute name="isAbstract">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
	</xsl:attribute-set>
	
	<!--


	-->
	<xsl:attribute-set name="messageAttributes">
		<xsl:attribute name="xmi.id">
			<xsl:value-of select="@id"/>
		</xsl:attribute>
		<xsl:attribute name="name">
			<!-- show the parallel label with the name if any -->
			<xsl:choose>
				<xsl:when test="string-length(@para-label) > 0">
					<xsl:value-of select="concat(@name,'{',@para-label,'}')"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="@name"/>
				</xsl:otherwise>
			</xsl:choose>
	
		</xsl:attribute>
		<xsl:attribute name="visibility">
			<xsl:value-of select="'public'"/>
		</xsl:attribute>
		<xsl:attribute name="isSpecification">
			<xsl:value-of select="'false'"/>
		</xsl:attribute>
		<xsl:attribute name="sender">
			<xsl:value-of select="concat('ClassifierRole_', @source-id)"/>
		</xsl:attribute>
		<xsl:attribute name="receiver">
			<xsl:value-of select="concat('ClassifierRole_', @destination-id)"/>
		</xsl:attribute>
		<xsl:attribute name="message3"/>
		<xsl:attribute name="predecessor"/>
		<xsl:attribute name="communicationConnection"/>
		<xsl:attribute name="action"/>
	</xsl:attribute-set>
</xsl:stylesheet>
<!-- end of xsl sheet -->