<?xml version="1.0" encoding="UTF-8"?>
<!--
/**
 * UCMExporter
 * @author Ali Echihabi
 *
 * (C) Copyright 2003 Use Case Maps User Group
 * www.usecasemaps.org
 */

  This style sheet takes an UCMExporterXML file and generates
  a description of the instances that will be used to add
  connector messages before entering and after leaving a parallel block (<par>)	

-->
<!-- start of xsl sheet -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xsltc="http://xml.apache.org/xalan/xsltc"
	extension-element-prefixes="xsltc" version="1.0">


<xsl:template match="/">
	<xsl:element name="source">
		<xsl:apply-templates select="//scenario"/>
	</xsl:element>
</xsl:template>


<xsl:template match="scenario">
<xsl:element name="scenarioDesc" use-attribute-sets="scenarioDescAttributes">
		<!-- detect locations for which we need to generate parallel message connectors -->
		<xsl:apply-templates select=".//paraMessageConnectors"/>
	</xsl:element>
</xsl:template>


<xsl:attribute-set name="scenarioDescAttributes">
	<xsl:attribute name="name">
		<xsl:value-of select="@name"/>
	</xsl:attribute>
</xsl:attribute-set>

<!--

this tempalte will put in the data about the parallel block message connectors

-->
<xsl:template match="paraMessageConnectors">
	<xsl:element name="par">
		<xsl:attribute name="id">
			<xsl:value-of select="@para-block-id"/>
		</xsl:attribute>
		<xsl:call-template name="generateParBlockInstanceInfo"/>
	</xsl:element>
</xsl:template>



<!--

This template will generate the information about this paralle block:
	- what are the last instance in each of its sequences
	- what is(are) the instance(s) that follow the <par> block immediately

-->
<xsl:template name="generateParBlockInstanceInfo">
	<xsl:call-template name="putPreviousBlockLastInstances"/>
	<xsl:call-template name="putFirstOwnInstances"/>
	<xsl:call-template name="putLastOwnInstances"/>
	<xsl:call-template name="putNextBlockFirstInstances"/>
</xsl:template>



<!--

This template identifies the set of instances that can be considered
as the last instances of the block immiediately preceding the current <par>
	- if it is a <do> then it is the instance
	- if it is a <par> then discard it. this information is the same as
	  the lastOwnInstances of the previous <par>
-->
<xsl:template name = "putPreviousBlockLastInstances">
	<xsl:element name="previousBlockInstances">
	<xsl:call-template name="putPreviousDoInstance"/>	
	</xsl:element>
</xsl:template>

<!--

this template sees if there is a <do> just before entering the <par>
In case ther is, we need to know the instance (object) of the <do>
in order to use it later on.

-->
<xsl:template name="putPreviousDoInstance">
		<xsl:for-each select="preceding-sibling::do[position()=1]">
			<xsl:element name="instance" use-attribute-sets="instanceAttributes"/>
		</xsl:for-each>
</xsl:template>


<!--

This template inserts the instance names of the FIRST <do> of each sequence
in the parallel block

-->

<xsl:template name="putFirstOwnInstances">
	<xsl:element name="firstOwnInstances">
		<xsl:for-each select="preceding-sibling::par[position() = 1]/seq/do[position()=1]">
			<xsl:element name="instance" use-attribute-sets="instanceAttributes"/>
		</xsl:for-each>
	</xsl:element>
</xsl:template>


<!--

This template inserts the instance names of the LAST <do> of each sequence
in the parallel block

-->
<xsl:template name="putLastOwnInstances">
	<xsl:element name="lastOwnInstances">
		<xsl:for-each select="preceding-sibling::par[position() = 1]/seq/do[last()]">
			<xsl:element name="instance" use-attribute-sets="instanceAttributes"/>		
		</xsl:for-each>
	</xsl:element>
</xsl:template>




<!--
this template will detecte the instance(s) that immediately follow
the current parallel block
-->
<xsl:template name="putNextBlockFirstInstances">
	<!-- this depends whether what follows is a <par> or <do>-->

	<!-- so we test which of the two comes first -->
	<xsl:element name="nextBlockInstances">
		<xsl:choose>
			<xsl:when test="name(following-sibling::*[name()='par' or name()='do'])='do'">
				<xsl:call-template name="putNextDoInstance"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="putNextParFirstInstances"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:element>

</xsl:template>


<!--
	this template puts the instance of the first <do> of each sequence
	of the following parallel block
-->

<xsl:template name="putNextParFirstInstances">
	<xsl:for-each select="following-sibling::par[position() = 1]/seq/do[position() = 1]">
		<xsl:element name="instance" use-attribute-sets="instanceAttributes"/>
	</xsl:for-each>

</xsl:template>

<!--
this template puts the instance of the first <do> that follows the parallel block
-->

<xsl:template name="putNextDoInstance">
	<xsl:for-each select="following-sibling::do[position()=1]">
		<xsl:element name="instance" use-attribute-sets="instanceAttributes"/>
	</xsl:for-each>
</xsl:template>




<xsl:attribute-set name="instanceAttributes">
	<xsl:attribute name="component-name">
		<xsl:value-of select="@component-name"/>
	</xsl:attribute>
	<xsl:attribute name="component-role">
		<xsl:value-of select="@component-role"/>
	</xsl:attribute>
</xsl:attribute-set>


</xsl:stylesheet>
<!-- end of xsl sheet -->