<!--

sample rule overrider.
Communication Protocol: PSIA (Please Say it Again)
Description: connector messages between all components use
			the PSIA protocol
Comments: Could be generalized to all messages.
		  Could specify that it is the case onl between
		  specific (component-name, role-name) pairs.

-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xsltc="http://xml.apache.org/xalan/xsltc"
	extension-element-prefixes="xsltc" version="1.0">
<xsl:include href="UCMExporterXMLDuplicator.xsl"/>	



<!--

So the example here does the following:
All connector messages must use the PSIA protocotl (see above).
We could add more restirictions such as only communication between
Instance I2 and I3 should use PSIA. Of course it is better to define such a rule
by specifying the instance component-name and role instance of the instance ID (Especially
that the user will have no clue what the instance ID will be until the UCMExporterXML is generated)
. In this case you will need to define a template that will take a component-name
and role and will return the instance ID. Look at the UCMExporterXML2MSC.xsl. It has a template
that does the oposite. It will help you writing your own.


-->
<xsl:template name="messageTemplate" match="message">
<!-- let's say we want to apply this only for connectors -->
	<xsl:choose>
		<xsl:when test="@is-connector = 'true'">
			<!-- we specify the id because TAU 4.4 is confused when two messages of the same name that are near to each other have the same id -->			
			<xsl:element name="message">
				<xsl:attribute name="id">
					<xsl:value-of select="concat('SIAP_ORIG',@id)"/>
				</xsl:attribute>
				<xsl:attribute name="name">
					<xsl:value-of select="@name"/>
				</xsl:attribute>
				<xsl:attribute name="source-id">
					<xsl:value-of select="@source-id"/>
				</xsl:attribute>
				<xsl:attribute name="destination-id">
					<xsl:value-of select="@destination-id"/>
				</xsl:attribute>
			</xsl:element>
			<xsl:element name="message">
				<xsl:attribute name="id">
					<xsl:value-of select="concat('SIAP',@id)"/>
				</xsl:attribute>
				<xsl:attribute name="name">
					<xsl:text>SAY_IT_AGAIN_PLEASE</xsl:text>
				</xsl:attribute>
				<xsl:attribute name="source-id">
					<xsl:value-of select="@destination-id"/>
				</xsl:attribute>
				<xsl:attribute name="destination-id">
					<xsl:value-of select="@source-id"/>
				</xsl:attribute>
			</xsl:element>
			<xsl:element name="message">
				<xsl:attribute name="id">
					<xsl:value-of select="concat('SIAP_REPLY',@id)"/>
				</xsl:attribute>
				<xsl:attribute name="name">
					<xsl:value-of select="@name"/>
				</xsl:attribute>
				<xsl:attribute name="source-id">
					<xsl:value-of select="@source-id"/>
				</xsl:attribute>
				<xsl:attribute name="destination-id">
					<xsl:value-of select="@destination-id"/>
				</xsl:attribute>
			</xsl:element>

		</xsl:when>
		<xsl:otherwise>
			<xsl:copy-of select="."/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>


<!-- the local atribute set overrieds the included one -->
<!-- it is not used in this example, but just to illustrate...-->
<xsl:attribute-set name="messageAttributeSet">
	<xsl:attribute name="local"></xsl:attribute>
</xsl:attribute-set>

</xsl:stylesheet>